@@ -16,26 +16,39 @@
 #'   a single location as a integer vector of size 1. This is suitable
 #'   for extracting with `[[`.
 #'
+#' * `num_as_location()` and `num_as_location2()` are specialized variants
+#'   that have extra options for numeric indices.
+#'
 #' @inheritParams vec_slice
 #' @inheritParams rlang::args_error_context
 #'
 #' @param n A single integer representing the total size of the
 #'   object that `i` is meant to index into.
+#'
 #' @param names If `i` is a character vector, `names` should be a character
 #'   vector that `i` will be matched against to construct the index. Otherwise,
 #'   not used. The default value of `NULL` will result in an error
 #'   if `i` is a character vector.
-#' @param missing Whether to throw an `"error"` when `i` is a missing
-#'   value, or `"propagate"` it (return it as is). By default, vector
-#'   subscripts can contain missing values and scalar subscripts can't.
-#' @param arg The argument name to be displayed in error messages when
-#'   `vec_as_location()` and `vec_as_location2()` are used to check the
-#'   type of a function input.
 #'
-#' @return `vec_as_location()` returns an integer vector that can be used
-#'   as an index in a subsetting operation. `vec_as_location2()`
-#'   returns an integer of size 1 that can be used a scalar index for
-#'   extracting an element.
+#' @param missing How should missing `i` values be handled?
+#'   - `"error"` throws an error.
+#'   - `"propagate"` returns them as is.
+#'   - `"remove"` removes them.
+#'
+#'   By default, vector subscripts propagate missing values but scalar
+#'   subscripts error on them.
+#'
+#'   Propagated missing values can't be combined with negative indices when
+#'   `negative = "invert"`, because they can't be meaningfully inverted.
+#'
+#' @param arg The argument name to be displayed in error messages.
+#'
+#' @return
+#' - `vec_as_location()` and `num_as_location()` return an integer vector that
+#'   can be used as an index in a subsetting operation.
+#'
+#' - `vec_as_location2()` and `num_as_location2()` return an integer of size 1
+#'   that can be used a scalar index for extracting an element.
 #'
 #' @examples
 #' x <- array(1:6, c(2, 3))
@@ -59,7 +72,7 @@ vec_as_location <- function(i,
                             n,
                             names = NULL,
                             ...,
-                            missing = c("propagate", "error"),
+                            missing = c("propagate", "remove", "error"),
                             arg = caller_arg(i),
                             call = caller_env()) {
   check_dots_empty0(...)
@@ -76,22 +89,35 @@ vec_as_location <- function(i,
     frame = environment()
   )
 }
+
 #' @rdname vec_as_location
-#' @param negative Whether to `"invert"` negative values to positive
-#'   locations, throw an informative `"error"`, or `"ignore"` them.
-#' @param oob If `"error"`, throws an informative `"error"` if some
-#'   elements are out-of-bounds. If `"extend"`, out-of-bounds
-#'   locations are allowed if they are consecutive after the end. This
-#'   can be used to implement extendable vectors like `letters[1:30]`.
-#' @param zero Whether to `"remove"` zero values, throw an informative
-#'   `"error"`, or `"ignore"` them.
+#'
+#' @param negative How should negative `i` values be handled?
+#'   - `"error"` throws an error.
+#'   - `"ignore"` returns them as is.
+#'   - `"invert"` returns the positive location generated by inverting the
+#'     negative location. When inverting, positive and negative locations
+#'     can't be mixed. This option is only applicable for `num_as_location()`.
+#'
+#' @param oob How should out-of-bounds `i` values be handled?
+#'   - `"error"` throws an error.
+#'   - `"remove"` removes both positive and negative out-of-bounds locations.
+#'   - `"extend"` allows positive out-of-bounds locations if they directly
+#'     follow the end of a vector. This can be used to implement extendable
+#'     vectors, like `letters[1:30]`.
+#'
+#' @param zero How should zero `i` values be handled?
+#'   - `"error"` throws an error.
+#'   - `"remove"` removes them.
+#'   - `"ignore"` returns them as is.
+#'
 #' @export
 num_as_location <- function(i,
                             n,
                             ...,
-                            missing = c("propagate", "error"),
+                            missing = c("propagate", "remove", "error"),
                             negative = c("invert", "error", "ignore"),
-                            oob = c("error", "extend"),
+                            oob = c("error", "remove", "extend"),
                             zero = c("remove", "error", "ignore"),
                             arg = caller_arg(i),
                             call = caller_env()) {
@@ -134,8 +160,6 @@ vec_as_location2 <- function(i,
   ))
 }
 #' @rdname vec_as_location
-#' @param negative Whether to throw an `"error"` when `i` is a
-#'   negative location value, or `"ignore"` it.
 #' @export
 num_as_location2 <- function(i,
                              n,
@@ -173,7 +197,6 @@ vec_as_location2_result <- function(i,
   result <- vec_as_subscript2_result(
     i = i,
     arg = arg,
-    logical = "error",
     call = call
   )
 
@@ -236,29 +259,24 @@ vec_as_location2_result <- function(i,
     )))
   }
 
-  # FIXME: Use result approach in internal implementation?
   err <- NULL
   i <- tryCatch(
-    vec_as_location(i, n, names = names, arg = arg),
-    vctrs_error_subscript_type = function(err) {
+    vec_as_location(i, n, names = names, arg = arg, call = call),
+    vctrs_error_subscript = function(err) {
+      err[["subscript_scalar"]] <- TRUE
       err <<- err
       i
     }
   )
+  if (!is_null(err)) {
+    return(result(err = err))
+  }
 
   if (neg) {
     i <- -i
   }
 
-  if (is_null(err)) {
-    result(i)
-  } else {
-    result(err = new_error_location2_type(
-      i = i,
-      subscript_arg = arg,
-      call = call
-    ))
-  }
+  result(i)
 }
 
 
@@ -323,7 +341,6 @@ new_error_location2_type <- function(i,
   new_error_subscript2_type(
     class = class,
     i = i,
-    logical = "error",
     numeric = "cast",
     character = "cast",
     ...
@@ -332,19 +349,19 @@ new_error_location2_type <- function(i,
 cnd_bullets_location2_need_scalar <- function(cnd, ...) {
   cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)
   format_error_bullets(c(
-    x = glue::glue_data(cnd, "{subscript_arg} has size {length(i)} but must be size 1.")
+    x = glue::glue_data(cnd, "{subscript_arg} must be size 1, not {length(i)}.")
   ))
 }
 cnd_bullets_location2_need_present <- function(cnd, ...) {
   cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)
   format_error_bullets(c(
-    x = glue::glue_data(cnd, "{subscript_arg} can't be `NA`.")
+    x = glue::glue_data(cnd, "{subscript_arg} must be a location, not {obj_type_friendly(i)}.")
   ))
 }
 cnd_bullets_location2_need_positive <- function(cnd, ...) {
   cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)
   format_error_bullets(c(
-    x = glue::glue_data(cnd, "{subscript_arg} has value {i} but must be a positive location.")
+    x = glue::glue_data(cnd, "{subscript_arg} must be a positive location, not {i}.")
   ))
 }
 
@@ -415,6 +432,31 @@ cnd_bullets_subscript_missing <- function(cnd, ...) {
   ))
 }
 
+stop_subscript_empty <- function(i, ..., call = caller_env()) {
+  cnd_signal(new_error_subscript_type(
+    i = i,
+    body = cnd_bullets_subscript_empty,
+    ...,
+    call = call
+  ))
+}
+cnd_bullets_subscript_empty <- function(cnd, ...) {
+  cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)
+
+  loc <- which(cnd$i == "")
+  if (length(loc) == 1) {
+    line <- glue::glue("It has an empty string at location {loc}.")
+  } else {
+    enum <- ensure_full_stop(enumerate(loc))
+    line <- glue::glue("It has an empty string at locations {enum}")
+  }
+
+  format_error_bullets(c(
+    x = glue::glue_data(cnd, "{subscript_arg} can't contain the empty string."),
+    x = line
+  ))
+}
+
 stop_indicator_size <- function(i, n, ..., call = caller_env()) {
   cnd_signal(new_error_subscript_size(
     i,
@@ -425,11 +467,10 @@ stop_indicator_size <- function(i, n, ..., call = caller_env()) {
   ))
 }
 cnd_body_vctrs_error_indicator_size <- function(cnd, ...) {
-  cnd$subscript_arg <- append_arg("subscript", cnd$subscript_arg)
+  cnd$subscript_arg <- append_arg("Logical subscript", cnd$subscript_arg)
   glue_data_bullets(
     cnd,
-    i = "Logical subscripts must match the size of the indexed input.",
-    x = "Input has size {n} but {subscript_arg} has size {vec_size(i)}."
+    x = "{subscript_arg} must be size 1 or {n}, not {vec_size(i)}."
   )
 }
 
@@ -456,7 +497,7 @@ cnd_header.vctrs_error_subscript_oob <- function(cnd, ...) {
   action <- cnd_subscript_action(cnd)
   type <- cnd_subscript_type(cnd)
 
-  if (action == "rename" || type == "character") {
+  if (action %in% c("rename", "relocate") || type == "character") {
     glue::glue("Can't {action} {elt[[2]]} that don't exist.")
   } else {
     glue::glue("Can't {action} {elt[[2]]} past the end.")
@@ -480,12 +521,17 @@ cnd_body.vctrs_error_subscript_oob <- function(cnd, ...) {
 cnd_body_vctrs_error_subscript_oob_location <- function(cnd, ...) {
   i <- cnd$i
 
-  # In case of negative indexing
-  i <- abs(i)
-
   # In case of missing locations
   i <- i[!is.na(i)]
 
+  if (cnd_subscript_action(cnd) == "negate") {
+    # Only report negative indices
+    i <- i[i < 0L]
+  }
+
+  # In case of negative indexing
+  i <- abs(i)
+
   oob <- i[i > cnd$size]
   oob_enum <- vctrs_cli_vec(oob)
 
@@ -559,6 +605,6 @@ cnd_body_vctrs_error_subscript_oob_non_consecutive <- function(cnd, ...) {
 
 cnd_subscript_oob_non_consecutive <- function(cnd) {
   out <- cnd$subscript_oob_non_consecutive %||% FALSE
-  stopifnot(is_bool(out))
+  check_bool(out)
   out
 }
