@@ -17,19 +17,20 @@ static
 r_obj* data_frame(r_obj* x,
                   r_ssize size,
                   const struct name_repair_opts* p_name_repair_opts,
-                  struct r_lazy call);
+                  struct r_lazy error_call);
 
 static
 r_obj* df_list(r_obj* x,
                r_ssize size,
+               bool unpack,
                const struct name_repair_opts* p_name_repair_opts,
-               struct r_lazy call);
+               struct r_lazy error_call);
 
 static
 r_obj* df_list_drop_null(r_obj* x);
 
 static
-r_obj* df_list_splice(r_obj* x);
+r_obj* df_list_unpack(r_obj* x);
 
 static
 void init_bare_data_frame(r_obj* x, r_ssize n);
