@@ -5,6 +5,7 @@
 #include "globals.h"
 
 r_ssize vec_size(r_obj* x);
+r_ssize vec_size_3(r_obj* x, struct vctrs_arg* p_arg, struct r_lazy call);
 
 r_obj* vec_check_recycle(r_obj* x,
                          r_ssize size,
@@ -19,7 +20,8 @@ r_obj* vec_recycle(r_obj* x,
 
 r_obj* vec_recycle_fallback(r_obj* x,
                             r_ssize size,
-                            struct vctrs_arg* x_arg);
+                            struct vctrs_arg* x_arg,
+                            struct r_lazy call);
 
 r_ssize df_size(r_obj* x);
 r_ssize df_raw_size(r_obj* x);
