@@ -1,4 +1,3 @@
-context("test-type-date-time")
 
 test_that("date-times have informative types", {
   expect_identical(vec_ptype_abbr(Sys.Date()), "date")
@@ -114,12 +113,9 @@ test_that("datetime coercions are symmetric and unchanging", {
   mat <- maxtype_mat(types)
 
   expect_true(isSymmetric(mat))
-  expect_known_output(
-    mat,
-    test_path("test-type-date-time.txt"),
-    print = TRUE,
-    width = 200
-  )
+
+  local_options(width = 200)
+  expect_snapshot(print(mat))
 })
 
 test_that("tz comes from first non-empty", {
@@ -177,6 +173,11 @@ test_that("vec_ptype2(<difftime>, NA) is symmetric (#687)", {
   )
 })
 
+test_that("vec_ptype2() standardizes duration storage type to double", {
+  x <- structure(1L, units = "secs", class = "difftime")
+  expect <- new_duration(double(), units = "secs")
+  expect_identical(vec_ptype2(x, x), expect)
+})
 
 # cast: dates ---------------------------------------------------------------
 
@@ -350,6 +351,17 @@ test_that("can cast NA and unspecified to duration", {
   expect_identical(vec_cast(unspecified(2), new_duration()), new_duration(dbl(NA, NA)))
 })
 
+test_that("casting coerces corrupt integer storage durations to double (#1602)", {
+  x <- structure(1L, units = "secs", class = "difftime")
+  expect <- new_duration(1, units = "secs")
+
+  expect_identical(vec_cast(x, x), expect)
+
+  # Names are retained through the coercion
+  names(x) <- "a"
+  expect_named(vec_cast(x, x), "a")
+})
+
 # proxy/restore: dates ---------------------------------------------------
 
 test_that("restoring an integer to an integer Date converts to double", {
